#include <cstdint>
#include <iostream>
#include <vector>

struct Context {
    Context() = default;
    int32_t id = -1;
    uint32_t switch_cost;
};

struct Mouse {
    Mouse() = default;
    Mouse(const Mouse&) = delete;
    Mouse(Mouse&& blub) = default;
    Mouse& operator=(const Mouse&) = delete;
    Mouse& operator=(Mouse&&) = default;
    uint32_t id;
    Context context;
    uint32_t dough_fetching_speed;
    uint32_t chocolate_fetching_speed;
    uint32_t lemon_fetching_speed;
    uint32_t bowl_filling_speed;
    uint32_t mixing_speed;
    uint32_t cup_filling_speed;
    uint32_t baking_speed;
};
    
struct Dough : Context {
    uint32_t max_amount;
};
    
struct Chocolate : Context {
    uint32_t max_amount;
};

struct Lemons : Context {
    uint32_t max_amount;
};

struct BowlFilling : Context{
};

struct Mixing : Context {
};
    
struct CupFilling : Context {
};

struct Baking : Context {
    uint32_t baking_time;
};

/**
 * Describes a bowl
 * capacity: Maximum amount of resources that can be filled into this bowl, keep in mind that for every piece of chocolate or lemon, also one piece of dough is added.
 * amount:   Current amount of the given resource inside the bowl
 * resource: The current resource inside the bowl, disregard if amount == 0
 * mixed:    Whether the bowl has already been mixed and is ready to be filled into cups
 * in_use:   Whether the bowl is currently being used by a mouse. This can be your own mouse or a different mouse.
 * id:       Bowl index
 */
struct Bowl {
    uint32_t capacity = 0;
    uint32_t amount = 0;
    uint32_t resource = 0;
    bool mixed = 0;
    bool in_use = 0;
    uint32_t id = 0;
};

/**
 * Describes an Order to be fulfilled.
 * tick:       The order must be fulfilled before or on this tick, otherwise it cannot be fulfilled. It can also be partially fulfilled.
 * muffin_type Type of muffins in this order, either 1: chocolate or 2: lemons
 * amount      Amount of muffins that this order requests.
 */
struct Order {
    uint32_t muffin_type;
    uint32_t tick;
    uint32_t amount;
};

/**
 * Storages stores information about the table, bowls and muffin cups
 * table: Vector with number of stored resources for each type
 *  table[0] = dough, table[1] = chocolate, table[2] = lemons
 * bowls: Vector of `Bowl` with current status of each bowl
 * cups: Vector with number of filled in muffin cups of each type:
 *  cups[1] = Number of filled cups with chocolate muffins,
 *  cups[2] = Number of filled cups with lemon muffins
 * max_cups: Integer denoting the maximum number of cups available.
 */
struct Storages {

    std::vector<uint32_t> table;
    std::vector<Bowl> bowls;
    std::vector<uint32_t> cups;
    uint32_t max_cups;
};
        
struct Contexts {
    Dough dough;
    Chocolate chocolate;
    Lemons lemons;
    BowlFilling bowl_filling;
    Mixing mixing;
    CupFilling cup_filling;
    Baking baking;
};

std::vector<Mouse> mice;
std::vector<Order> orders;
Storages storages;
Contexts contexts;
uint32_t my_id;
uint32_t current_tick;
uint32_t current_turn;



void mouse_going_to_dough_station(const Mouse& mouse, uint32_t arrival_tick);

void mouse_going_to_chocolate_station(const Mouse& mouse, uint32_t arrival_tick);

void mouse_going_to_lemon_station(const Mouse& mouse, uint32_t arrival_tick);

void mouse_going_to_bowl_filling_station(const Mouse& mouse, uint32_t arrival_tick);

void mouse_going_to_mixing_station(const Mouse& mouse, uint32_t arrival_tick);

void mouse_going_to_cup_filling_station(const Mouse& mouse, uint32_t arrival_tick);

void mouse_going_to_baking_station(const Mouse& mouse, uint32_t arrival_tick);

void mouse_getting_dough(const Mouse& mouse, uint32_t finish_tick);

void mouse_getting_chocolate(const Mouse& mouse, uint32_t finish_tick);

void mouse_getting_lemons(const Mouse& mouse, uint32_t finish_tick);

void mouse_filling_bowls(const Mouse& mouse, Bowl bowl, uint32_t resource_id, const Storages& old_storages, uint32_t finish_tick);

void mouse_mixing_bowl(const Mouse& mouse, Bowl bowl, uint32_t finish_tick);

void mouse_filling_cups(const Mouse& mouse, Bowl bowl, const Storages& old_storages, uint32_t finish_tick);

void mouse_baking_muffins(const Mouse& mouse, const Storages& old_storages, uint32_t finish_tick);

void mouse_idling(const Mouse& mouse, uint32_t finish_tick);
                  
void print_mouse_stats();
                  
void print_orders();
                  
void print_storages();
                  
void make_muffins();


namespace blub {

void update_containers() {
    for (uint32_t i = 0; i < 3; i++) {
        std::cin >> storages.table[i];
    }
    for (Bowl& bowl : storages.bowls) {
        std::cin >> bowl.amount >> bowl.resource >> bowl.mixed >> bowl.in_use;
    }
    std::cin >> storages.cups[1] >> storages.cups[2];
}

void check(bool blub, const std::string& complaint) {
    if (!blub) {
        std::cerr << complaint << std::endl;
        std::exit(1);
    }
}

void process_events() {

    std::string eventType;
    std::cin >> eventType;
    if (eventType == "T") {
        std::cin >> current_tick >> current_turn;
        
        if (current_turn == my_id) {
            update_containers();
            return;
        }
        process_events();
    }
    else if (eventType == "A") {
        uint32_t action;
        std::cin >> action;
        Mouse& mouse = mice[current_turn];
        Storages old_storages(storages);

        if (action == 0) {
            // switch context
            uint32_t context_id, busy_until;
            std::cin >> context_id >> busy_until;
            update_containers();
            switch(context_id) {
            case 0:
                mouse_going_to_dough_station(mouse, busy_until);
                mouse.context = contexts.dough;
                break;
            case 1:
                mouse_going_to_chocolate_station(mouse, busy_until);
                mouse.context = contexts.chocolate;
                break;
            case 2:
                mouse_going_to_lemon_station(mouse, busy_until);
                mouse.context = contexts.lemons;
                break;
            case 3:
                mouse_going_to_bowl_filling_station(mouse, busy_until);
                mouse.context = contexts.bowl_filling;
                break;
            case 4:
                mouse_going_to_mixing_station(mouse, busy_until);
                mouse.context = contexts.mixing;
                break;
            case 5:
                mouse_going_to_cup_filling_station(mouse, busy_until);
                mouse.context = contexts.cup_filling;
                break;
            case 6:
                mouse_going_to_baking_station(mouse, busy_until);
                mouse.context = contexts.baking;
                break;
            }
        }        
        if (action == 1) {
            // execute context
            
            int32_t context_id = mouse.context.id;
            uint32_t finish_tick;
            if (context_id == 0) {
                std::cin >> finish_tick;
                update_containers();
                mouse_getting_dough(mouse, finish_tick);
            }
            if (context_id == 1) {
                std::cin >> finish_tick;
                update_containers();
                mouse_getting_chocolate(mouse, finish_tick);
            }
            if (context_id == 2) {
                std::cin >> finish_tick;
                update_containers();
                mouse_getting_lemons(mouse, finish_tick);
            }
            if (context_id == 3) {
                
                uint32_t bowl_id, resource_id;
                std::cin >> bowl_id >> resource_id >> finish_tick;
                Bowl& bowl = storages.bowls[bowl_id];
                update_containers();
                mouse_filling_bowls(mouse, bowl, resource_id, old_storages, finish_tick);
            }
            if (context_id == 4) {
                uint32_t bowl_id;
                std::cin >> bowl_id >> finish_tick;
                Bowl& bowl = storages.bowls[bowl_id];
                update_containers();
                mouse_mixing_bowl(mouse, bowl, finish_tick);
            }
            if (context_id == 5) {
                uint32_t bowl_id;
                std::cin >> bowl_id >> finish_tick;
                Bowl& bowl = storages.bowls[bowl_id];
                update_containers();
                mouse_filling_cups(mouse, bowl, old_storages, finish_tick);
            }
            if (context_id == 6) {
                std::cin >> finish_tick;
                update_containers();
                mouse_baking_muffins(mouse, old_storages, finish_tick);
            }
        }
        if (action == 2) {
            // idle
            int32_t finish_tick;
            std::cin >> finish_tick;
            update_containers();
            mouse_idling(mouse, finish_tick);
        }
        process_events();
    }
    else if (eventType == "E") {
        std::exit(0);
    } else {
        blub::check(false, "Unknown input '" +  eventType + "' (" + std::to_string(eventType.size()) + ")");
    }
}
} //namespace blub
        
void idle(int32_t until = -1) {
    std::cout << 2 << " " << until << std::endl;
    blub::process_events();
}
        
void get_dough() {
    blub::check(mice[my_id].context.id == contexts.dough.id, "Can't get dough when not at dough station");
    std::cout << 1 << std::endl;
    blub::process_events();
}
        
void get_chocolate() {
    blub::check(mice[my_id].context.id == contexts.chocolate.id, "Can't get chocolate when not at chocolate station");
    std::cout << 1 << std::endl;
    blub::process_events();
}
        
void get_lemons() {
    blub::check(mice[my_id].context.id == contexts.lemons.id, "Can't get lemons when not at lemon station");
    std::cout << 1 << std::endl;
    blub::process_events();
}
        
void fill_bowl(const Bowl& bowl, uint32_t resource_id) {
    blub::check(mice[my_id].context.id == contexts.bowl_filling.id, "Can't fill bowls when not at bowl filling station");
    blub::check(bowl.id < storages.bowls.size(), "Invalid bowl id " + std::to_string(bowl.id) + ", it must be in [0, " + std::to_string(storages.bowls.size() - 1) + "]");
    std::cout << 1 << " " <<  bowl.id << " " <<  resource_id << std::endl;
    blub::process_events();
}
        
void mix_bowl(const Bowl& bowl) {
    blub::check(mice[my_id].context.id == contexts.mixing.id, "Can't mix bowls when not at mixing station");
    blub::check(bowl.id < storages.bowls.size(), "Invalid bowl id " + std::to_string(bowl.id) + ", it must be in [0, " + std::to_string(storages.bowls.size() - 1) + "]");
    std::cout << 1 << " " << bowl.id << std::endl;
    blub::process_events();
}
        
void fill_cups(const Bowl& bowl) {
    blub::check(mice[my_id].context.id == contexts.cup_filling.id, "Can't fill cups when not at cup filling station");
    blub::check(bowl.id < storages.bowls.size(), "Invalid bowl id " + std::to_string(bowl.id) + ", it must be in [0, " + std::to_string(storages.bowls.size() - 1) + "]");
    std::cout << 1 << " " << bowl.id << std::endl;
    blub::process_events();
}
        
void bake_muffins() {
    blub::check(mice[my_id].context.id == contexts.baking.id, "Can't bake muffins when not at baking station");
    std::cout << 1 << std::endl;
    blub::process_events();
}
        
void go_to(const Context& context) {
    std::cout << 0 << " " << context.id << std::endl;
    blub::process_events();
}
        
int main() {

    current_tick = 0;
    uint32_t num_bowls, capacity;
    std::cin >> num_bowls >> capacity;
    std::cin >> storages.max_cups;
    for (uint32_t i = 0; i < num_bowls; i++) {
        storages.bowls.emplace_back();
        storages.bowls.back().capacity = capacity;
        storages.bowls.back().id = i;
    }
    storages.cups.resize(3, 0);
    storages.table.resize(3, 0);
    std::cin >> contexts.dough.switch_cost >> contexts.dough.max_amount;
    std::cin >> contexts.chocolate.switch_cost >> contexts.chocolate.max_amount;
    std::cin >> contexts.lemons.switch_cost >> contexts.lemons.max_amount;
    std::cin >> contexts.bowl_filling.switch_cost;
    std::cin >> contexts.mixing.switch_cost;
    std::cin >> contexts.cup_filling.switch_cost;
    std::cin >> contexts.baking.switch_cost >> contexts.baking.baking_time;
    contexts.dough.id = 0;
    contexts.chocolate.id = 1;
    contexts.lemons.id = 2;
    contexts.bowl_filling.id = 3;
    contexts.mixing.id = 4;
    contexts.cup_filling.id = 5;
    contexts.baking.id = 6;

    uint32_t num_mice;
    std::cin >> num_mice;
    for (uint32_t i = 0; i < num_mice; i++) {
        mice.emplace_back();
        mice.back().id = i;
        std::cin >> mice.back().dough_fetching_speed >> mice.back().chocolate_fetching_speed >> mice.back().lemon_fetching_speed >> mice.back().bowl_filling_speed >> mice.back().mixing_speed >> mice.back().cup_filling_speed >> mice.back().baking_speed;
    }
    std::cin >> my_id;
    uint32_t num_orders;
    std::cin >> num_orders;
    for (uint32_t i = 0; i < num_orders; i++) {
        orders.emplace_back();
        std::cin >> orders.back().muffin_type >> orders.back().tick >> orders.back().amount;
    }
    blub::process_events();
    make_muffins();
}        
        
